#include "stdafx.h"
#include "WaveEd.h"
#include "MainFrm.h"
#include "renderdialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNAMIC(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_SETFOCUS()
	ON_COMMAND(ID_FILE_NEW, OnFileNew)
	ON_COMMAND(ID_FILE_LOAD, OnFileLoad)
	ON_COMMAND(ID_FILE_CLOSE, OnFileClose)
	ON_COMMAND(ID_VST_EDIT, OnVstEdit)
	ON_UPDATE_COMMAND_UI(ID_FILE_CLOSE, OnUpdateFileClose)
	ON_COMMAND(ID_VIEW_TOOLBAR2, OnViewToolbar2)
	ON_UPDATE_COMMAND_UI(ID_VIEW_TOOLBAR2, OnUpdateViewToolbar2)
	ON_UPDATE_COMMAND_UI(ID_BUTTON_PLAY, OnUpdateButtonPlay)
	ON_COMMAND(ID_BUTTON_PLAY, OnButtonPlay)
	ON_UPDATE_COMMAND_UI(ID_BUTTON_STOP, OnUpdateButtonStop)
	ON_COMMAND(ID_BUTTON_STOP, OnButtonStop)
	ON_COMMAND(ID_VIEW_TOOLBAR3, OnViewToolbar3)
	ON_UPDATE_COMMAND_UI(ID_VIEW_TOOLBAR3, OnUpdateViewToolbar3)
	ON_COMMAND(ID_BUTTON_PAUSE, OnButtonPause)
	ON_UPDATE_COMMAND_UI(ID_BUTTON_PAUSE, OnUpdateButtonPause)
	ON_UPDATE_COMMAND_UI(ID_FILE_RENDER, OnUpdateFileRender)
	ON_COMMAND(ID_FILE_RENDER, OnFileRender)
	ON_COMMAND(ID_VST_REMOVE, OnVstRemove)
	ON_UPDATE_COMMAND_UI(ID_VST_REMOVE, OnUpdateVstRemove)
	ON_UPDATE_COMMAND_UI(ID_VST_EDIT, OnUpdateVstEdit)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

CMainFrame::CMainFrame()
{
	char CurrentDir[MAX_PATH];
	inMenuPlugPos = -1;

	 pPluginHost = new VSTPlugin(AfxGetInstanceHandle(), &Engine);
	 pPlayer = new WavePlayer(pPluginHost);

	while ( !(pPlayer->InitDevice(2048, 16, 2, 44100, 16)) )
	{
		 if ( (AfxMessageBox(IDS_MSG_AUDIOFAILED,
		MB_RETRYCANCEL | MB_APPLMODAL | MB_ICONERROR) == IDCANCEL)) break;
	}

	GetCurrentDirectory(MAX_PATH-10, CurrentDir);
	lstrcat(CurrentDir, "\\vstplugs\\");
	 
	pPluginHost->SetConfig(44100, 2048, CurrentDir);
	pPluginHost->EnumPlugins();

	m_pOpenRequester = new CFileDialog(true, "wav", NULL, 
								OFN_ENABLESIZING | OFN_FILEMUSTEXIST,
								"Wave files (*.wav)|*.wav|", this);
}

CMainFrame::~CMainFrame()
{
	delete m_pOpenRequester;
	pPlayer->StopPlaying();
	delete pPlayer;
	delete pPluginHost;
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	// create a view to occupy the client area of the frame
	if (!m_wndView.Create(NULL, NULL, AFX_WS_DEFAULT_VIEW,
		CRect(0, 0, 0, 0), this, AFX_IDW_PANE_FIRST, NULL))
	{
		TRACE0("Failed to create view window\n");
		return -1;
	}
	m_wndView.AttachEngine(&Engine, pPlayer);
	
	m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);
	m_wndToolBar.LoadToolBar(IDR_MAINFRAME);
	
	m_wndNavToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);
	m_wndNavToolBar.LoadToolBar(IDR_NAVIGATE);
	
	m_wndZoomToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);
	m_wndZoomToolBar.LoadToolBar(IDR_ZOOMBAR);

	m_wndStatusBar.Create(this);
	m_wndStatusBar.SetIndicators(indicators, sizeof(indicators)/sizeof(UINT));
	
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	m_wndNavToolBar.EnableDocking(CBRS_ALIGN_ANY);
	m_wndZoomToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);
	DockControlBarLeftOf(&m_wndNavToolBar, &m_wndToolBar);
	DockControlBarLeftOf(&m_wndZoomToolBar, &m_wndNavToolBar);

	m_wndWavesMenu.Attach(GetWavesPopupHandle());
	m_wndVSTMenu.Attach(GetVSTPopupHandle());
	FillVSTMenu();

	return 0;
}

void CMainFrame::DockControlBarLeftOf(CToolBar* Bar,CToolBar* LeftOf)
{
	CRect rect;
	DWORD dw;
	UINT n;

	RecalcLayout();
	LeftOf->GetWindowRect(&rect);
	rect.OffsetRect(1,0);
	dw=LeftOf->GetBarStyle();
	n = 0;
	n = (dw&CBRS_ALIGN_TOP) ? AFX_IDW_DOCKBAR_TOP : n;
	n = (dw&CBRS_ALIGN_BOTTOM && n==0) ? AFX_IDW_DOCKBAR_BOTTOM : n;
	n = (dw&CBRS_ALIGN_LEFT && n==0) ? AFX_IDW_DOCKBAR_LEFT : n;
	n = (dw&CBRS_ALIGN_RIGHT && n==0) ? AFX_IDW_DOCKBAR_RIGHT : n;

	DockControlBar(Bar,n,&rect);
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;
	cs.dwExStyle &= ~WS_EX_CLIENTEDGE;
	cs.lpszClass = AfxRegisterWndClass(0);
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers
void CMainFrame::OnSetFocus(CWnd* pOldWnd)
{
	// forward focus to the view window
	m_wndView.SetFocus();
}

BOOL CMainFrame::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
	if (pHandlerInfo == NULL)
	{
		if ((nID >= ID_VST_MENUITEM_000) && (nID <= ID_VST_MENUITEM_000 + 255))
		{
			if (nCode == CN_COMMAND)
			{
				return OnVSTMenu(nID, nCode, pExtra, pHandlerInfo);
			}
			else if (nCode == CN_UPDATE_COMMAND_UI)
			{
				((CCmdUI*)pExtra)->Enable(TRUE);
				((CCmdUI*)pExtra)->SetCheck(inMenuPlugPos == (nID - ID_VST_MENUITEM_000));
				return true;
			}
		}
	}
	
	if (m_wndView.OnCmdMsg(nID, nCode, pExtra, pHandlerInfo)) return true;

	return CFrameWnd::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}


void CMainFrame::OnFileNew() 
{
	if (AfxMessageBox(IDS_MSG_FLUSHDATA, MB_OKCANCEL | MB_ICONQUESTION, 0) == IDOK)
	{
		pPlayer->StopPlaying();
		pPlayer->SetLooping(false);
		Engine.New();

		//Clear Waves menu.
		for (int i = m_wndWavesMenu.GetMenuItemCount()-1; i>=0;i--)
		{
			m_wndWavesMenu.RemoveMenu(i, MF_BYPOSITION);
		}
		m_wndWavesMenu.AppendMenu(MF_STRING, ID_WAVESPOPUP_01, "No files loaded");

		m_wndView.ResetView(-1); //-1, czyli workspace!
		Invalidate(true);
	}
}

void CMainFrame::OnFileLoad() 
{
	bool WasEmpty = Engine.IsEmpty();
	m_wndView.DoNotMove();
	if (m_pOpenRequester->DoModal() == IDOK)
	{
		if (!Engine.LoadWave(m_pOpenRequester->GetPathName()))
		{
			AfxMessageBox(IDS_MSG_WAVELOADFAILED, MB_OK | MB_ICONSTOP, 0);

			return;
		}
	
		if (WasEmpty) m_wndWavesMenu.RemoveMenu(0, MF_BYPOSITION);
		m_wndWavesMenu.AppendMenu(MF_STRING,
							  ID_WAVESELECT_000 + m_wndWavesMenu.GetMenuItemCount(),
							  m_pOpenRequester->GetFileName());
	
		m_wndView.ResetView(m_wndWavesMenu.GetMenuItemCount()-1);
	}
_ASSERT(_CrtCheckMemory());
}

HMENU CMainFrame::GetWavesPopupHandle()
{
  	HMENU tmpPopup;
	CMenu tmpMenu;
	
	tmpMenu.Attach(*(GetMenu()));
	tmpPopup = (tmpMenu.GetSubMenu(3))->m_hMenu;
	tmpMenu.Detach();
	tmpMenu.Attach(tmpPopup);
	tmpPopup = (tmpMenu.GetSubMenu(5))->m_hMenu;
	tmpMenu.Detach();
	return tmpPopup;
}


HMENU CMainFrame::GetVSTPopupHandle()
{
	HMENU tmpPopup;
	CMenu tmpMenu;

	tmpMenu.Attach(*(GetMenu()));
	tmpPopup = (tmpMenu.GetSubMenu(2))->m_hMenu;
	tmpMenu.Detach();
	return tmpPopup;
}

void CMainFrame::OnFileClose() 
{
	// TODO: Add your command handler code here
	
}

void CMainFrame::FillVSTMenu()
{
	int i = 0;
	char* pPlugName;

	if (pPluginHost->GetPluginName(0))
	{
		//m_wndVSTMenu.RemoveMenu(0, MF_BYPOSITION);

		while (pPlugName = pPluginHost->GetPluginName(i))
		{			
			m_wndVSTMenu.AppendMenu(MF_STRING, ID_VST_MENUITEM_000 + i, pPlugName);
			DrawMenuBar();
			i++;
			//Zdefiniowalem 256 wpisow w resource.h, wiec mozna zaladowac 256 pluginow.
			//Chyba wystarczy...
		}
	}
}

bool CMainFrame::OnVSTMenu(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
	pPluginHost->unLoad(0, 0);
	//A wiec na razie mozna wstawic tylko jeden plugin ;(((
	//Ale to sie moze szybko zmienic, bo engine w pelni obsluguje kolejke pluginow!
	inMenuPlugPos = nID - ID_VST_MENUITEM_000;
	if (!pPluginHost->Load(pPluginHost->GetPluginDllName(inMenuPlugPos), 0, VST_LOAD, 0))
	{
		AfxMessageBox(IDS_MSG_PLUGINFAILED, MB_ICONSTOP | MB_OK, 0);
	}

	pPluginHost->OpenPluginDialog(0, inMenuPlugPos);
	return true;
}

void CMainFrame::OnVstEdit() 
{
	pPluginHost->OpenPluginDialog(0, inMenuPlugPos);
}

void CMainFrame::OnUpdateFileClose(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(false);	
}

void CMainFrame::OnUpdateButtonPlay(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!Engine.IsEmpty() && m_wndView.isWorkspaceDisplayed() && !pPlayer->_isPlaying());
}

void CMainFrame::OnButtonPlay() 
{
	pPlayer->StopPlaying();
	if (m_wndView.GetSelectionLength())
	{
		pPlayer->StartPlaying(m_wndView.GetSelectionStart(),
							  m_wndView.GetSelectionLength() + m_wndView.GetSelectionStart());
	}

	else pPlayer->StartPlaying(m_wndView.GetDispStart(),
							   m_wndView.GetDispStart() + m_wndView.GetDispLength());
}

void CMainFrame::OnUpdateButtonStop(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!Engine.IsEmpty());
}

void CMainFrame::OnButtonStop() 
{
	pPlayer->StopPlaying();
}

void CMainFrame::OnViewToolbar2() 
{
	m_wndNavToolBar.ShowWindow((m_wndNavToolBar.GetStyle() & WS_VISIBLE) == 0);
	RecalcLayout();
}

void CMainFrame::OnUpdateViewToolbar2(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((m_wndNavToolBar.GetStyle() & WS_VISIBLE) != 0);
}

void CMainFrame::OnViewToolbar3() 
{
	m_wndZoomToolBar.ShowWindow((m_wndZoomToolBar.GetStyle() & WS_VISIBLE) == 0);
	RecalcLayout();	
}

void CMainFrame::OnUpdateViewToolbar3(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((m_wndZoomToolBar.GetStyle() & WS_VISIBLE) != 0);	
}
void CMainFrame::OnButtonPause() 
{
	if (pPlayer->isPaused()) pPlayer->Resume();
	else pPlayer->Pause();
}

void CMainFrame::OnUpdateButtonPause(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!Engine.IsEmpty());
}

void CMainFrame::OnUpdateFileRender(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(Engine.GetWorkspaceSize());
}

void CMainFrame::OnFileRender() 
{
	CRenderDialog dialog;
	dialog.SetupPointers(pPluginHost, &Engine);
	pPlayer->StopPlaying();
	dialog.DoModal();
}

void CMainFrame::OnVstRemove() 
{
	pPluginHost->unLoad(0, 0);
	inMenuPlugPos = -1;
	
}

void CMainFrame::OnUpdateVstRemove(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!pPluginHost->IsEmpty());
	
}

void CMainFrame::OnUpdateVstEdit(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!pPluginHost->IsEmpty());
}
